import { recentWeekdays, rangeCount } from '@/utils/weekday/index'
const dayTimeStamp = 8.64e7
/**
 * 将时间戳转成年月日时分秒
 * @param {(Object|string|number)} time
 * @returns {Date Object}
 */
export function timeTrans(time) {
  let date
  if (typeof time === 'object') {
    date = time
  } else {
    if (typeof time === 'string') {
      if (/^[0-9]+$/.test(time)) {
        // support "1548221490638"
        time = parseInt(time)
      } else {
        // support safari
        // https://stackoverflow.com/questions/4310953/invalid-date-in-safari
        time = time.replace(new RegExp(/-/gm), '/')
      }
    }

    if (typeof time === 'number' && time.toString().length === 10) {
      time = time * 1000
    }
    date = new Date(time)
  }
  return date
}

/**
 * 获取一天内从某小时至某小时结束的时间点
 * @param {number} start
 * @param {number} end
 * @param {number} step (60分钟/30分钟/15分钟)
 * @returns {Date Object}
 */
export function getTimeDots(start, end, step) {
  const arr = []
  start = parseInt(start)
  end = parseInt(end)
  for (let i = start; i <= end; i++) {
    arr.push({
      time: i < 10 ? '0' + i + ':00' : i + ':00'
    })
    if (i < 24 && step === 15) {
      arr.push({
        time: i < 10 ? '0' + i + ':15' : i + ':15'
      })
    }
    if (i < 24 && (step === 30 || step === 15)) {
      arr.push({
        time: i < 10 ? '0' + i + ':30' : i + ':30'
      })
    }
    if (i < 24 && step === 15) {
      arr.push({
        time: i < 10 ? '0' + i + ':45' : i + ':45'
      })
    }
  }
  return arr
}
/**
 * 获取当前date时间对应的半小时
 * @param {string} type ['start', 'end']
 * @param {Date Object} date
 * @returns {string}
 */
export function getCurHalfHour(type, date) {
  const day = date || new Date();
  let h = day.getHours()
  const m = day.getMinutes();
  let time = ''
  if (m === 0) {
    h < 10 && (h = '0' + h)
    time = type === 'start' ? h + ':00' : h + ':30'
  } else if (m <= 30) {
    const _h = h + 1 < 10 ? '0' + (h + 1) : h + 1;
    h < 10 && (h = '0' + h)
    time = type === 'start' ? h + ':30' : _h + ':00'
  } else {
    const _h = h + 1 < 10 ? '0' + (h + 1) : h + 1;
    time = type === 'start' ? _h + ':00' : _h + ':30'
  }
  return time
}
/**
 * 获取当前date时间对应的半小时的日期
 * @param {string} type ['start', 'end']
 * @param {Date Object} date
 * @returns {Date Object}
 */
export function getCurHalfHourDate(type, date) {
  date = date ? timeTrans(date) : new Date()
  const time = getCurHalfHour(type, date)
  const hmArr = time.split(':')
  const h = parseInt(hmArr[0])
  const m = parseInt(hmArr[1])
  return new Date(date.getFullYear(), date.getMonth(), date.getDate(), h, m, 0)
}
/**
 * 将日期转换为年月日整时
 * @param {(Object|string|number)} date
 * @returns {Date Object}
 */
export function getYMDByDate(date) {
  date = timeTrans(date)
  return new Date(date.getFullYear(), date.getMonth(), date.getDate())
}
/**
 * 将日期转换为年月日整时
 * @param {(Object|string|number)} date
 * @returns {Date Object}
 */
export function getTimestampYMD(date) {
  return getYMDByDate(date).getTime()
}
/**
 * 获取从起始日之后的N个月范围的日期
 * @param {(Object|string|number)} startDate
 * @param {number} times
 * @returns {Date Object}
 */
function getDateByMouths(startDate, times) {
  startDate = timeTrans(startDate);
  let year = startDate.getFullYear();
  let month = startDate.getMonth() + 1;
  const day = startDate.getDate();
  let targetMDays = 30;
  let targetDate = null
  // startDate.setMonth(month + times)
  if ((month + times - 1) / 12 > 1) {
    month = (month + times - 1) % 12
    year += Math.floor((month + times - 1) / 12)
  } else {
    month = month + times - 1
  }
  targetMDays = new Date(year, month, 0).getDate()
  if (targetMDays < day) {
    targetDate = new Date(year, month - 1, targetMDays)
  } else {
    targetDate = new Date(year, month - 1, day)
  }
  return targetDate
}
/**
 * 获取从起始日至结束日之间的月数
 * @param {(Object|string|number)} startDate
 * @param {number} times
 * @returns {Date Object}
 */
function getMonthsByDate(startDate, endDate) {
  let year = startDate.getFullYear();
  let month = startDate.getMonth() + 1;
  let monthDays = new Date(year, month, 0).getDate();
  let times = 1;
  const rangeTimeStamp = getTimestampYMD(endDate) - getTimestampYMD(startDate);
  let totalDays = rangeTimeStamp / dayTimeStamp;
  while (totalDays >= monthDays) {
    totalDays -= monthDays;
    ++month
    ++times
    if (month / 12 > 1) {
      month = month % 12
      ++year
    }
    monthDays = new Date(year, month, 0).getDate();
  }
  return times
}
/**
 * 将时间戳转成年月日时分秒
 * @param {(Object|string|number)} time
 * @param {string} cFormat
 * @returns {string | null}
 */
export function dateFormat(time, cFormat) {
  if (arguments.length === 0 || !time) {
    return null
  }
  const format = cFormat || '{y}-{m}-{d} {h}:{i}:{s}'
  const date = timeTrans(time)
  const formatObj = {
    y: date.getFullYear(),
    m: date.getMonth() + 1,
    d: date.getDate(),
    h: date.getHours(),
    i: date.getMinutes(),
    s: date.getSeconds(),
    a: date.getDay()
  }
  const timeStr = format.replace(/{([ymdhisa])+}/g, (result, key) => {
    const value = formatObj[key]
    // Note: getDay() returns 0 on Sunday
    if (key === 'a') {
      return ['日', '一', '二', '三', '四', '五', '六'][value]
    }
    return value.toString().padStart(2, '0')
  })
  return timeStr
}

/**
 * 周期会议通过重复频率和重复次数计算结束日期
 * @param {(Object|string|number)} startDate
 * @param {number} times
 * @param {string} repeatRate
 * @returns {string | null}
 */
export function computedDateByRateTimes(startDate, times, repeatRate) {
  let rangeTimeStamp = 0
  let endTime
  startDate = timeTrans(startDate)
  switch (repeatRate) {
    case 0: {
      rangeTimeStamp = (times - 1) * dayTimeStamp
      endTime = Date.parse(startDate) + rangeTimeStamp
      break
    }
    case 1: {
      const dateArr = recentWeekdays(times, startDate)
      endTime = dateArr[dateArr.length - 1]
      break
    }
    case 2: {
      rangeTimeStamp = (times - 1) * 7 * dayTimeStamp
      endTime = Date.parse(startDate) + rangeTimeStamp
      break
    }
    case 3: {
      rangeTimeStamp = (times - 1) * 14 * dayTimeStamp
      endTime = Date.parse(startDate) + rangeTimeStamp
      break
    }
    case 4: {
      endTime = getDateByMouths(startDate, times)
      break
    }
  }
  const endDate = dateFormat(endTime, '{y}-{m}-{d}')
  return endDate
}
/**
 * 判断两个date是否是同一天
 * @param {(Object|string|number)} startDate
 * @param {(Object|string|number)} endDate
 * @returns {Boolean}
 */
export function isSameDate(startDate, endDate) {
  const start = timeTrans(startDate)
  const end = timeTrans(endDate)
  return dateFormat(start, '{y}-{m}-{d}') === dateFormat(end, '{y}-{m}-{d}')
}
/**
 * 周期会议通过重复频率和起、始日期计算重复次数
 * @param {(Object|string|number)} startDate
 * @param {(Object|string|number)} endDate
 * @param {string} repeatRate
 * @returns {string | null}
 */
export function computedTimesByRateDate(startDate, endDate, repeatRate) {
  const rangeTimeStamp = getTimestampYMD(endDate) - getTimestampYMD(startDate)
  let times = 0
  switch (repeatRate) {
    case 0: {
      times = rangeTimeStamp / dayTimeStamp + 1
      break
    }
    case 1: {
      times = rangeCount(timeTrans(startDate), timeTrans(endDate))
      break
    }
    case 2: {
      times = rangeTimeStamp / (dayTimeStamp * 7) + 1
      break
    }
    case 3: {
      times = rangeTimeStamp / (dayTimeStamp * 14) + 1
      break
    }
    case 4: {
      times = getMonthsByDate(startDate, endDate)
      break
    }
  }
  return times
}
